import bpy
from mathutils import Color

#-----------------------------------

class NOXTextureSlotProps(bpy.types.PropertyGroup):
    texID = bpy.props.IntProperty(
        name = "ID",
        description = "ID of instance in textures array.",
        default = -1,
        min = -1,
        max = 1000
    )
    brightness = bpy.props.FloatProperty(
        name = "Brightness",
        description = "Brightness of texture",
        default = 0.0,
        min = -1.0,
        max = 1.0
    )
    contrast = bpy.props.FloatProperty(
        name = "Contrast",
        description = "Contrast of texture",
        default = 1.0,
        min = 0.0,
        max = 4.0
    )
    saturation = bpy.props.FloatProperty(
        name = "Saturation",
        description = "Saturation of texture",
        default = 0.0,
        min = -1.0,
        max = 1.0
    )
    hue = bpy.props.FloatProperty(
        name = "Hue",
        description = "Hue of texture",
        default = 0.0,
        min = 0.0,
        max = 1.0
    )
    red = bpy.props.FloatProperty(
        name = "Red",
        description = "Add red to texture",
        default = 0.0,
        min = -1.0,
        max = 1.0
    )
    green = bpy.props.FloatProperty(
        name = "Green",
        description = "Add green to texture",
        default = 0.0,
        min = -1.0,
        max = 1.0
    )
    blue = bpy.props.FloatProperty(
        name = "Blue",
        description = "Add blue to texture",
        default = 0.0,
        min = -1.0,
        max = 1.0
    )
    gamma = bpy.props.FloatProperty(
        name = "Gamma",
        description = "Gamma of texture. Recommended: 2.2 for colors and 1.0 for values.",
        default = 2.2,
        min = 1.0,
        max = 4.0
    )
    invert = bpy.props.BoolProperty(
        name = "Invert", 
        description = "Invert texture colors", 
        default = False
    )
    bilinear = bpy.props.BoolProperty(
        name = "Bilinear interpolation", 
        description = "Interpolates value from near pixels. Useful for small textures.", 
        default = False
    )
    enabled = bpy.props.BoolProperty(
        name = "Enabled", 
        description = "Enable using texture in current slot.", 
        default = False
    )

#-----------------------------------

def updateNOXcolor(self, context):
    if context.material != None:
        if hasattr(context.material, "noxData"):
            if context.material.noxData.layers.__len__()>0:
                if context.material.noxData.layers[0].emission:
                    context.material.diffuse_color = context.material.noxData.layers[0].emission_color
                    context.material.specular_color = (0.0, 0.0, 0.0)
                else:
                    context.material.diffuse_color = context.material.noxData.layers[0].refl0
                    context.material.specular_color = context.material.noxData.layers[0].refl90
    
#-----------------------------------

class NOXMatLayerProps(bpy.types.PropertyGroup):
    weight = bpy.props.FloatProperty(
        name = "Weight",
        description = "Layer contribution",
        default = 100.0,
        min = 0.0,
        max = 100.0
    )
    refl0 = bpy.props.FloatVectorProperty(
        name = "Reflection 0",
        description = "Reflection for 0",
        default = (0.8, 0.8, 0.8),
        subtype = 'COLOR',
        min = 0.0,
        max = 1.0,
        size = 3,
        update=updateNOXcolor
    )
    refl90 = bpy.props.FloatVectorProperty(
        name = "Reflection 90",
        description = "Reflection for 90",
        default = (0.9, 0.9, 0.9),
        subtype = 'COLOR',
        min = 0.0,
        max = 1.0,
        size = 3,
        update=updateNOXcolor
    )
    rough = bpy.props.FloatProperty(
        name = "Roughness",
        description = "Layer roughness",
        default = 100.0,
        min = 0.0,
        max = 100.0
    )
    ior = bpy.props.FloatProperty(
        name = "IOR",
        description = "Index of refraction",
        default = 1.6,
        min = 1.0,
        max = 100.0
    )
    transmission = bpy.props.BoolProperty(
        name = "Transmission", 
        description = "Transmission enabled", 
        default = False
    )
    absorption = bpy.props.BoolProperty(
        name = "Absorption", 
        description = "Absorption enabled", 
        default = False
    )
    absorption_color = bpy.props.FloatVectorProperty(
        name = "",
        description = "Absorption color",
        default = (0.9, 0.9, 0.9),
        subtype = 'COLOR',
        min = 0.0,
        max = 1.0,
        size = 3
    )
    absorption_distance = bpy.props.FloatProperty(
        name = "Distance",
        description = "Absorption distance",
        default = 10.0,
        min = 0.0,
        max = 100.0
    )
    dispersion = bpy.props.BoolProperty(
        name = "Dispersion", 
        description = "Dispersion enabled", 
        default = False
    )
    dispersion_size = bpy.props.FloatProperty(
        name = "Size",
        description = "Dispersion size",
        default = 1.0,
        min = 0.0,
        max = 1.0
    )
    fake_glass = bpy.props.BoolProperty(
        name = "Fake glass", 
        description = "Fake glass enabled", 
        default = False
    )
    emission = bpy.props.BoolProperty(
        name = "Emission", 
        description = "Emission enabled", 
        default = False,
        update=updateNOXcolor
    )
    emission_color = bpy.props.FloatVectorProperty(
        name = "Emission color",
        description = "Emission color",
        default = (1.0, 1.0, 1.0),
        subtype = 'COLOR',
        min = 0.0,
        max = 1.0,
        size = 3,
        update=updateNOXcolor
    )
    emission_power = bpy.props.FloatProperty(
        name = "Emission power",
        description = "Emission power",
        default = 100.0,
        min = 0.0,
        max = 10000.0
    )
    emission_unit = bpy.props.EnumProperty(
        items = [("W", "W", "Watt"), 
            ("Wm", "W/m2", "Watt per square meter"), 
            ("lm", "lm", "Lumen"),
            ("lx", "lx", "Lux = lumen per square meter")
            ],
        name = "Unit",
        description = "Emission power unit"
    )
    emission_temperature = bpy.props.IntProperty(
        name = "Temperature",
        description = "Temperature of blackbody emitter.",
        default = 6500,
        min = 1000,
        max = 10000
    )
    ies_file = bpy.props.StringProperty(
        name = "IES",
        description = "IES profile file",
        subtype = "FILE_PATH"
    )


    textures = bpy.props.EnumProperty(
        items = [("refl0", "Reflection 0", "Reflection 0"), 
            ("refl90", "Reflection 90", "Reflection 90"), 
            ("weight", "Weight", "Weight"),
            ("rough", "Roughness", "Roughness"),
            ("light", "Emission", "Emission"),
            ("normal", "Normal", "Normal"),
            ("opacity", "Opacity", "Opacity")
            ],
        name = "Edited texture",
        description = "Currently edited texture slot"
    )
    
    tex_refl0 = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    tex_refl90 = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    tex_rough = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    tex_weight = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    tex_light = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    tex_normal = bpy.props.PointerProperty(type=NOXTextureSlotProps)

#-----------------------------------

class NOXMaterialProps(bpy.types.PropertyGroup):
    layers = bpy.props.CollectionProperty(
        type = NOXMatLayerProps,
        name = "Material layers",
        description = "Material layers list"
    )
    lay_index = bpy.props.IntProperty(
        min = 0,
        max = 15,
        default = 0
    )
    preview_id = bpy.props.IntProperty(
        min = -1,
        default = -1
    ) 

    opacity = bpy.props.FloatProperty(
        name = "Opacity",
        description="0 for invisible, 100 for opaque",
        default = 100.0,
        min = 0.0,
        max = 100.0
    )

    tex_opacity = bpy.props.PointerProperty(type=NOXTextureSlotProps)
    opacity_texture = bpy.props.StringProperty(
        name = "Opacity Texture",
        description="Texture for opacity",
        default = ""
    )

    skyportal = bpy.props.BoolProperty(
        name = "Skyportal", 
        description = "Material is skyportal. Do not use together with emission in layers.", 
        default = False
    )

    blend_layer = bpy.props.EnumProperty(
        items = [("b1", "Layer 1", "Blend layer 1"), 
            ("b2", "Layer 2", "Blend layer 2"), 
            ("b3", "Layer 3", "Blend layer 3"), 
            ("b4", "Layer 4", "Blend layer 4"), 
            ("b5", "Layer 5", "Blend layer 5"), 
            ("b6", "Layer 6", "Blend layer 6"), 
            ("b7", "Layer 7", "Blend layer 7"), 
            ("b8", "Layer 8", "Blend layer 8"), 
            ("b9", "Layer 9", "Blend layer 9"), 
            ("b10", "Layer 10", "Blend layer 10"), 
            ("b11", "Layer 11", "Blend layer 11"), 
            ("b12", "Layer 12", "Blend layer 12"), 
            ("b13", "Layer 13", "Blend layer 13"), 
            ("b14", "Layer 14", "Blend layer 14"), 
            ("b15", "Layer 15", "Blend layer 15"), 
            ("b16", "Layer 16", "Blend layer 16")
            ],
        name = "Blend layer",
        description = "Blend layer for emitter"
    )

#-----------------------------------

	
 